from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractNetworks, Network


class Networks(AbstractNetworks):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def get(self, *, network_id: str) -> Network:
        json_payload = {}

        if network_id is not None:
            json_payload["network_id"] = network_id

        res = self.client.post("/networks/get", json=json_payload)

        return Network.from_dict(res["network"])

    def list(
        self,
    ) -> List[Network]:
        json_payload = {}

        res = self.client.post("/networks/list", json=json_payload)

        return [Network.from_dict(item) for item in res["networks"]]



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractBridges


class Bridges(AbstractBridges):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def get(self, *, bridge_id: str) -> None:
        json_payload = {}

        if bridge_id is not None:
            json_payload["bridge_id"] = bridge_id

        self.client.post("/bridges/get", json=json_payload)

        return None

    def list(
        self,
    ) -> None:
        json_payload = {}

        self.client.post("/bridges/list", json=json_payload)

        return None



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractThermostatsSchedules, ThermostatSchedule


class ThermostatsSchedules(AbstractThermostatsSchedules):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def create(
        self,
        *,
        climate_preset_key: str,
        device_id: str,
        ends_at: str,
        starts_at: str,
        is_override_allowed: Optional[bool] = None,
        max_override_period_minutes: Optional[int] = None,
        name: Optional[str] = None
    ) -> ThermostatSchedule:
        json_payload = {}

        if climate_preset_key is not None:
            json_payload["climate_preset_key"] = climate_preset_key
        if device_id is not None:
            json_payload["device_id"] = device_id
        if ends_at is not None:
            json_payload["ends_at"] = ends_at
        if starts_at is not None:
            json_payload["starts_at"] = starts_at
        if is_override_allowed is not None:
            json_payload["is_override_allowed"] = is_override_allowed
        if max_override_period_minutes is not None:
            json_payload["max_override_period_minutes"] = max_override_period_minutes
        if name is not None:
            json_payload["name"] = name

        res = self.client.post("/thermostats/schedules/create", json=json_payload)

        return ThermostatSchedule.from_dict(res["thermostat_schedule"])

    def delete(self, *, thermostat_schedule_id: str) -> None:
        json_payload = {}

        if thermostat_schedule_id is not None:
            json_payload["thermostat_schedule_id"] = thermostat_schedule_id

        self.client.post("/thermostats/schedules/delete", json=json_payload)

        return None

    def get(self, *, thermostat_schedule_id: str) -> ThermostatSchedule:
        json_payload = {}

        if thermostat_schedule_id is not None:
            json_payload["thermostat_schedule_id"] = thermostat_schedule_id

        res = self.client.post("/thermostats/schedules/get", json=json_payload)

        return ThermostatSchedule.from_dict(res["thermostat_schedule"])

    def list(
        self, *, device_id: str, user_identifier_key: Optional[str] = None
    ) -> List[ThermostatSchedule]:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id
        if user_identifier_key is not None:
            json_payload["user_identifier_key"] = user_identifier_key

        res = self.client.post("/thermostats/schedules/list", json=json_payload)

        return [
            ThermostatSchedule.from_dict(item) for item in res["thermostat_schedules"]
        ]

    def update(
        self,
        *,
        thermostat_schedule_id: str,
        climate_preset_key: Optional[str] = None,
        ends_at: Optional[str] = None,
        is_override_allowed: Optional[bool] = None,
        max_override_period_minutes: Optional[int] = None,
        name: Optional[str] = None,
        starts_at: Optional[str] = None
    ) -> None:
        json_payload = {}

        if thermostat_schedule_id is not None:
            json_payload["thermostat_schedule_id"] = thermostat_schedule_id
        if climate_preset_key is not None:
            json_payload["climate_preset_key"] = climate_preset_key
        if ends_at is not None:
            json_payload["ends_at"] = ends_at
        if is_override_allowed is not None:
            json_payload["is_override_allowed"] = is_override_allowed
        if max_override_period_minutes is not None:
            json_payload["max_override_period_minutes"] = max_override_period_minutes
        if name is not None:
            json_payload["name"] = name
        if starts_at is not None:
            json_payload["starts_at"] = starts_at

        self.client.post("/thermostats/schedules/update", json=json_payload)

        return None



from typing import Dict, Optional, Union
import time

from ..client import SeamHttpClient
from ..exceptions import SeamActionAttemptFailedError, SeamActionAttemptTimeoutError
from ..routes.models import ActionAttempt

TIMEOUT = 5.0
POLLING_INTERVAL = 0.5


def get_action_attempt(client: SeamHttpClient, action_attempt_id: str) -> ActionAttempt:
    res = client.post(
        "/action_attempts/get", json={"action_attempt_id": action_attempt_id}
    )

    return ActionAttempt.from_dict(res["action_attempt"])


def poll_until_ready(
    client: SeamHttpClient,
    *,
    action_attempt_id: str,
    timeout: Optional[float] = TIMEOUT,
    polling_interval: Optional[float] = POLLING_INTERVAL
) -> ActionAttempt:
    time_waiting = 0.0

    action_attempt = get_action_attempt(client, action_attempt_id)

    while action_attempt.status == "pending":
        time.sleep(polling_interval)
        time_waiting += polling_interval

        if time_waiting > timeout:
            raise SeamActionAttemptTimeoutError(action_attempt, timeout)

        action_attempt = get_action_attempt(client, action_attempt_id)

    if action_attempt.status == "error":
        raise SeamActionAttemptFailedError(action_attempt)

    return action_attempt


def resolve_action_attempt(
    client: SeamHttpClient,
    *,
    action_attempt: ActionAttempt,
    wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]]
) -> ActionAttempt:
    if wait_for_action_attempt is True:
        return poll_until_ready(
            client=client, action_attempt_id=action_attempt.action_attempt_id
        )

    if isinstance(wait_for_action_attempt, dict):
        return poll_until_ready(
            client=client,
            action_attempt_id=action_attempt.action_attempt_id,
            timeout=wait_for_action_attempt.get("timeout", TIMEOUT),
            polling_interval=wait_for_action_attempt.get(
                "polling_interval", POLLING_INTERVAL
            ),
        )

    return action_attempt



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractWebhooks, Webhook


class Webhooks(AbstractWebhooks):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def create(self, *, url: str, event_types: Optional[List[str]] = None) -> Webhook:
        json_payload = {}

        if url is not None:
            json_payload["url"] = url
        if event_types is not None:
            json_payload["event_types"] = event_types

        res = self.client.post("/webhooks/create", json=json_payload)

        return Webhook.from_dict(res["webhook"])

    def delete(self, *, webhook_id: str) -> None:
        json_payload = {}

        if webhook_id is not None:
            json_payload["webhook_id"] = webhook_id

        self.client.post("/webhooks/delete", json=json_payload)

        return None

    def get(self, *, webhook_id: str) -> Webhook:
        json_payload = {}

        if webhook_id is not None:
            json_payload["webhook_id"] = webhook_id

        res = self.client.post("/webhooks/get", json=json_payload)

        return Webhook.from_dict(res["webhook"])

    def list(
        self,
    ) -> List[Webhook]:
        json_payload = {}

        res = self.client.post("/webhooks/list", json=json_payload)

        return [Webhook.from_dict(item) for item in res["webhooks"]]

    def update(self, *, event_types: List[str], webhook_id: str) -> None:
        json_payload = {}

        if event_types is not None:
            json_payload["event_types"] = event_types
        if webhook_id is not None:
            json_payload["webhook_id"] = webhook_id

        self.client.post("/webhooks/update", json=json_payload)

        return None



from typing import Optional, Any, List, Dict, Union
from ..client import SeamHttpClient
from .models import AbstractAcsEncodersSimulate


class AcsEncodersSimulate(AbstractAcsEncodersSimulate):
    def __init__(self, client: SeamHttpClient, defaults: Dict[str, Any]):
        self.client = client
        self.defaults = defaults

    def next_credential_encode_will_fail(
        self,
        *,
        acs_encoder_id: str,
        error_code: Optional[str] = None,
        acs_credential_id: Optional[str] = None
    ) -> None:
        json_payload = {}

        if acs_encoder_id is not None:
            json_payload["acs_encoder_id"] = acs_encoder_id
        if error_code is not None:
            json_payload["error_code"] = error_code
        if acs_credential_id is not None:
            json_payload["acs_credential_id"] = acs_credential_id

        self.client.post(
            "/acs/encoders/simulate/next_credential_encode_will_fail", json=json_payload
        )

        return None

    def next_credential_encode_will_succeed(
        self, *, acs_encoder_id: str, scenario: Optional[str] = None
    ) -> None:
        json_payload = {}

        if acs_encoder_id is not None:
            json_payload["acs_encoder_id"] = acs_encoder_id
        if scenario is not None:
            json_payload["scenario"] = scenario

        self.client.post(
            "/acs/encoders/simulate/next_credential_encode_will_succeed",
            json=json_payload,
        )

        return None

    def next_credential_scan_will_fail(
        self,
        *,
        acs_encoder_id: str,
        error_code: Optional[str] = None,
        acs_credential_id_on_seam: Optional[str] = None
    ) -> None:
        json_payload = {}

        if acs_encoder_id is not None:
            json_payload["acs_encoder_id"] = acs_encoder_id
        if error_code is not None:
            json_payload["error_code"] = error_code
        if acs_credential_id_on_seam is not None:
            json_payload["acs_credential_id_on_seam"] = acs_credential_id_on_seam

        self.client.post(
            "/acs/encoders/simulate/next_credential_scan_will_fail", json=json_payload
        )

        return None

    def next_credential_scan_will_succeed(
        self,
        *,
        acs_encoder_id: str,
        acs_credential_id_on_seam: Optional[str] = None,
        scenario: Optional[str] = None
    ) -> None:
        json_payload = {}

        if acs_encoder_id is not None:
            json_payload["acs_encoder_id"] = acs_encoder_id
        if acs_credential_id_on_seam is not None:
            json_payload["acs_credential_id_on_seam"] = acs_credential_id_on_seam
        if scenario is not None:
            json_payload["scenario"] = scenario

        self.client.post(
            "/acs/encoders/simulate/next_credential_scan_will_succeed",
            json=json_payload,
        )

        return None
